<?php
/*
Plugin Name: Comvatars
Plugin URI: http://soeren-weber.net/post/2005/08/07/44
Description: Shows avatar images on user comments to uniquely identify the author much like various bulletin board systems do. The <a href="http://www.skippy.net/blog/2005/03/24/gravatars/">Gravatars plugin by Skippy</a> and the <a href="http://dev.wp-plugins.org/wiki/favatars">Favatars plugin by Jeff Minard</a> must be installed and activated to achieve the full functionality. Please visit the plugin homepage for further documentation.
Version: 2.3
Author: S&ouml;ren Weber
Author URI: http://soeren-weber.net/
*/

//error_reporting(E_ALL);

/* the current version number of this file; MUST be identical to the above
documented 'Version:' comment */
define('COMVATAR_VERSION', '2.3');

/* in case we haven't an email address we set a default to force
gravatars.com to execute our request; this will not be displayed */
define('COMVATAR_ANONYMOUS_EMAIL', 'force_gravatar@example.com');

/* <img /> tags created by this plugin will get this HTML class to allow
further CSS styling of the image; this can be overridden most functions;
if the default implementation is used, the class name will be appended by
'-default' every time a hook function outputs an <img /> tag */
define('COMVATAR_CSS_CLASS', 'comvatar');

/* the size of the avatar images as they appear in the admin panel of WP */
define('COMVATAR_ADMIN_ICON_SIZE', 16);

/* unique plugin id used for loading localized text and storing options
in the database */
define('COMVATAR_PLUGIN_ID', 'comvatars');

/* this is the path to the manager page relative to the wordpress installation */
define('COMVATAR_MANAGER_PAGE', 'wp-comvatars-manager.php');

/* the default limit as from the commenter is allowed to declare his own
avatar */
define('COMVATAR_DEFAULT_FREQUENT_AUTHOR_LIMIT', 10);

/* the minimum amount of images in the pool that have to be unregistered */
define('COMVATAR_DEFAULT_UNREGISTERED_AVATAR_LIMIT', 1);

/* the user_level since when a user can browse the registration database */
define('COMVATAR_DEFAULT_USER_LEVEL', 8);

/* html parts, that can be shown idependently by the manager */
define('COMVATAR_MANAGER_SHOW_DISPLAY_HEADER',       0x00000001);
define('COMVATAR_MANAGER_SHOW_DISPLAY_ERROR',        0x00000002);
define('COMVATAR_MANAGER_SHOW_DISPLAY_INFO',         0x00000004);
define('COMVATAR_MANAGER_SHOW_SEARCH_REGISTRATIONS', 0x00000008);
define('COMVATAR_MANAGER_SHOW_DISPLAY_AVATAR',       0x00000010);
define('COMVATAR_MANAGER_SHOW_CHANGE_AVATAR',        0x00000020);

// --------------------------------------------------------------------------
// The ComvatarManager class, displays and handles the avatar management page
// --------------------------------------------------------------------------

class ComvatarManager
{
	var $m_manager_page_url = '';
	var $m_user_message = '';
	var $m_show_page_part = 0;
	var $m_comment_comvatar_url = false;

	// ------------------------------------------------------------------------
	// initialization
	// ------------------------------------------------------------------------

	function ComvatarManager()
	{
		comvatar_init();
		// depending on what's the name of this page, we determine
		// if we are called from inside the admin panel and if called
		// outside whether to load the admin style sheet
		$this->m_manager_page_url = strrev($_SERVER['SCRIPT_NAME']);
		if (($r = strpos($this->m_manager_page_url, strrev('wp-admin/edit.php')))
			!== false && $r == 0)
			$this->m_manager_page_url = get_settings('siteurl').
				'/wp-admin/edit.php?page='. COMVATAR_PLUGIN_ID. '.php';
		else if (($r = strpos($this->m_manager_page_url, strrev(COMVATAR_MANAGER_PAGE)))
			!== false && $r == 0)
		{
			$this->m_manager_page_url = get_settings('siteurl'). '/'. COMVATAR_MANAGER_PAGE;
			$this->m_show_page_part |= COMVATAR_MANAGER_SHOW_DISPLAY_HEADER;
		}
		else
			$this->m_manager_page_url = get_permalink();
	}

	// ------------------------------------------------------------------------
	// helper stuff
	// ------------------------------------------------------------------------

	function is_selected_as_avatar($file)
	{
		if ($this->m_comment_comvatar_url != $file)
			return false;
		return true;
	}

	function is_avatar($address)
	{
		global $g_comvatar;

		if (empty($address))
			return true;
		$pos = strrpos($address, '/');
		if ($pos === false)
			return false;
		$address_path = substr($address, 0, $pos+1);
		if ($address_path != get_settings('siteurl'). '/'.
			$g_comvatar->m_registered_pool_path)
			return false;
		return true;
}

	function set_message($action, $message)
	{
		$this->m_show_page_part &=
			COMVATAR_MANAGER_SHOW_DISPLAY_HEADER
			+ COMVATAR_MANAGER_SHOW_SEARCH_REGISTRATIONS
			+ COMVATAR_MANAGER_SHOW_DISPLAY_AVATAR
			+ COMVATAR_MANAGER_SHOW_CHANGE_AVATAR;
		$this->m_show_page_part |= $action;
		$this->m_user_message = $message;
	}

	function create_key($email, $action)
	{
		global $g_comvatar;
		return $g_comvatar->create_key($email, $action);
	}

	function send_mail($to, $subject, $message)
	{
		$subject = '[' . get_bloginfo('name') . '] ' . $subject;
		$headers = "From: ". get_bloginfo('name').	" <". get_settings('admin_email'). ">\n";
		$headers .=	"Content-Type: text/plain; charset=\"" . get_settings('blog_charset') . "\"\n";
		return wp_mail($to, $subject, $message, $headers);
	}

	// ------------------------------------------------------------------------
	// execution functions for the different page actions
	// ------------------------------------------------------------------------

	function execute_change_avatar_feedback()
	{
		if (!isset($_REQUEST['avatar']))
		{
			return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
				__("You must specify an avatar. Don't link directly to this site.", COMVATAR_PLUGIN_ID));
		}
		else if (!$this->is_avatar($_REQUEST['avatar']))
		{
			return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
				sprintf(__("Avatar address '%s' is not allowed.", COMVATAR_PLUGIN_ID),
					$_REQUEST['avatar']));
		}

		$first = '?';
		if (strpos($this->m_manager_page_url, '?'))
			$first = '&;';
		$link = $this->m_manager_page_url.
			$first. 'action=change_avatar_confirmation'.
			'&email='.	$_REQUEST['email'].
			'&key='. $this->create_key($_REQUEST['email'], 'change_avatar_confirmation').
			'&avatar='. $_REQUEST['avatar'];
		$status = $this->send_mail($_REQUEST['email'],
			__('Avatar change confirmation', COMVATAR_PLUGIN_ID),
			sprintf(__("You are receiving this automated mail, because you have selected to change your avatar on this site. If this wasn't the case, simply ignore this e-mail. Otherwise click on the link below to confirm and activate the change.\n\n%s", COMVATAR_PLUGIN_ID),
				$link));

		if (!$status)
		{
			return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
				__("The confirmation mail could not be sent. Avatar wasn't changed.", COMVATAR_PLUGIN_ID));
		}
		return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_INFO,
			sprintf(__("A confirmation e-mail was sent to '%s'. The new avatar will be activated after you have followed the provided link in the confirmation e-mail.", COMVATAR_PLUGIN_ID),
				$_REQUEST['email']));
	}

	function execute_change_avatar_confirmation()
	{
		global $wpdb, $g_comvatar;

		if (!isset($_REQUEST['avatar']))
		{
			return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
				__("You must specify an avatar. Don't link directly to this site.", COMVATAR_PLUGIN_ID));
		}
		else if (!$this->is_avatar($_REQUEST['avatar']))
		{
			return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
				sprintf(__("Avatar address '%s' is not allowed.", COMVATAR_PLUGIN_ID),
					$_REQUEST['avatar']));
		}

		if (!empty($_REQUEST['avatar']))
		{
			$query = "
				SELECT COUNT(*)
				FROM `$wpdb->comments`
				WHERE `comment_author_email` != '". $_REQUEST['email']. "'
				AND `comment_comvatar_url` = '". $_REQUEST['avatar']. "'
			";
			$count = $wpdb->get_var($query);
			if ($count > 0)
			{
				return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
					sprintf(__("Avatar '%s' is already used by some other user.", COMVATAR_PLUGIN_ID),
						$_REQUEST['avatar']));
			}

			$limit = $g_comvatar->m_unregistered_avatar_limit;
			if (!empty($this->m_comment_comvatar_url))
				$limit -= 1;
			if ($g_comvatar->get_num_of_unregistered($g_comvatar->m_registered_pool_files)
				<= $limit)
			{
				return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
					sprintf(__("Sorry but no more avatars are currently available for registration. You may ask the site administrator to add some more avatars to the pool.", COMVATAR_PLUGIN_ID),
						$_REQUEST['avatar']));
			}
		}

		$query = "
			UPDATE `$wpdb->comments`
			SET `comment_comvatar_url` = '". $_REQUEST['avatar']. "'
			WHERE `comment_author_email` = '". $_REQUEST['email']. "'
		";
		$wpdb->query($query);
		$this->m_comment_comvatar_url = $_REQUEST['avatar'];
		$g_comvatar->init_pool($g_comvatar->m_anonymous_pool_files,
			$g_comvatar->m_anonymous_pool_path);
		$g_comvatar->init_pool($g_comvatar->m_identified_pool_files,
			$g_comvatar->m_identified_pool_path);
		$g_comvatar->init_pool($g_comvatar->m_registerd_pool_files,
			$g_comvatar->m_registered_pool_path);

		return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_INFO,
			__("Avatar successfully changed.", COMVATAR_PLUGIN_ID));
	}

	function execute_action()
	{
		get_currentuserinfo();
		global $g_comvatar, $wpdb, $user_level;

		// the admin is the boss and allowed to see all registrations
		if ($user_level >= $g_comvatar->m_admin_user_level)
			$this->m_show_page_part |= COMVATAR_MANAGER_SHOW_SEARCH_REGISTRATIONS;

		// check action parameter
		if (!isset($_REQUEST['action']) || empty($_REQUEST['action']))
		{
			if ($user_level < $g_comvatar->m_admin_user_level)
			{
				return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
					__("You must specify an action. Don't link directly to this site.", COMVATAR_PLUGIN_ID));
			}
			// the admin can access the page without an action, so we select a default
			$_REQUEST['action'] = 'display_avatar';
		}

		// check email parameter
		if (!isset($_REQUEST['email']) || empty($_REQUEST['email']))
		{
			if ($user_level < $g_comvatar->m_admin_user_level
				|| (isset($_REQUEST['email']) && empty($_REQUEST['email'])))
			{
				return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
					__('No e-mail address provided.', COMVATAR_PLUGIN_ID));
			}
		}
		else if (!is_email($_REQUEST['email']))
		{
			return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
				sprintf(__("E-mail address '%s' has invalid format.", COMVATAR_PLUGIN_ID),
					$_REQUEST['email']));
		}

		// check key parameter
		if (!isset($_REQUEST['key']) || empty($_REQUEST['key']))
		{
			if ($user_level < $g_comvatar->m_admin_user_level
				|| $_REQUEST['action'] != 'display_avatar')
			{
				return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
					__("You must specify a valid key. Don't link directly to this site.", COMVATAR_PLUGIN_ID));
			}
		}
		else if ($_REQUEST['key']
			!= $this->create_key($_REQUEST['email'], $_REQUEST['action']))
		{
			return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
				__('The specified key is invalid.', COMVATAR_PLUGIN_ID));
		}

		// set internals if we have an email address
		if (isset($_REQUEST['email']) && !empty($_REQUEST['email'])
			&& is_email($_REQUEST['email']))
		{
			$query = "
				SELECT `comment_comvatar_url`, COUNT(*) `count`
				FROM `$wpdb->comments`
				WHERE `comment_author_email` = '". $_REQUEST['email']. "'
				AND `comment_approved` = '1'
				GROUP BY `comment_comvatar_url`
			";
			$result = $wpdb->get_results($query);
			if ($user_level < $g_comvatar->m_admin_user_level && empty($result))
			{
				return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
					sprintf(__("A user with the e-mail address '%s' does not exist.", COMVATAR_PLUGIN_ID),
						$_REQUEST['email']));
			}
			if ($user_level < $g_comvatar->m_admin_user_level
				&& $result[0]->count < $g_comvatar->m_frequent_author_limit)
			{
				// this is not a hard error, we still want to display users avatar
				$this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
					sprintf(__("Sorry, you are not recognized as frequent comment author. Until now you have written %s comments but you need at least %s to set your avatar. Just type some more comments and come back.", COMVATAR_PLUGIN_ID),
						$result[0]->count, $g_comvatar->m_frequent_author_limit));
				$_REQUEST['action'] = 'display_avatar';
			}

			$this->m_show_page_part |= COMVATAR_MANAGER_SHOW_DISPLAY_AVATAR;
			if ($user_level < $g_comvatar->m_admin_user_level || !empty($result))
			{
				$this->m_comment_comvatar_url = trim($result[0]->comment_comvatar_url);
				$this->m_show_page_part |= COMVATAR_MANAGER_SHOW_CHANGE_AVATAR;
			}
		}

		// execute the action
		if ($_REQUEST['action'] == 'display_avatar')
			return;
		else if ($_REQUEST['action'] == 'change_avatar_feedback')
			return $this->execute_change_avatar_feedback();
		else if ($_REQUEST['action'] == 'change_avatar_confirmation')
			return $this->execute_change_avatar_confirmation();
		return $this->set_message(COMVATAR_MANAGER_SHOW_DISPLAY_ERROR,
			sprintf(__("Unknown action '%s'. Don't link directly to this site.", COMVATAR_PLUGIN_ID),
				$_REQUEST['action']));
	}

// --------------------------------------------------------------------------
// printing out html parts
// --------------------------------------------------------------------------

	function show_message()
	{
		$class_name = 'updated';
		if ($this->m_show_page_part & COMVATAR_MANAGER_SHOW_DISPLAY_ERROR)
			$class_name = COMVATAR_CSS_CLASS. '-error';
		if ($this->m_show_page_part & (COMVATAR_MANAGER_SHOW_DISPLAY_ERROR
			+ COMVATAR_MANAGER_SHOW_DISPLAY_INFO))
		{
?>
<div class='<?php echo $class_name; ?>'>
	<p><strong><?php echo $this->m_user_message; ?></strong></p>
</div>
<?php
		}
	}

	function show_search_registrations()
	{
		if ($this->m_show_page_part & COMVATAR_MANAGER_SHOW_SEARCH_REGISTRATIONS)
		{
?>
<div class="wrap">
	<h2><?php _e('Comvatars Manager', COMVATAR_PLUGIN_ID); ?></h2>
	<fieldset class="options">
		<legend><?php _e('Find Registration', COMVATAR_PLUGIN_ID); ?></legend>
		<form name="search_registrations" method="post" action="<?php echo $this->m_manager_page_url; ?>">
			<input type="hidden" name="action" value="display_avatar" />
			<p>
				<?php _e('Enter a comment authors e-mail address to view its registrated avatar.', COMVATAR_PLUGIN_ID); ?>

			</p>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('E-mail address:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<input name="email" type="text" id="email" size="40" />
					</td>
				</tr>
			</table>
			<p class="submit">
				<input type="submit" value="<?php _e('Search registrations &raquo;', COMVATAR_PLUGIN_ID); ?>" />
			</p>
		</form>
	</fieldset>
</div>
<?php
		}
	}

	function show_display_avatar()
	{
		global $g_comvatar;

		if (!empty($this->m_comment_comvatar_url))
			$avatar = $this->m_comment_comvatar_url;
		else
		{
			$g_comvatar->get_string_type($_REQUEST['email'],
				$user_name, $user_email, $user_url);
			$avatar = $g_comvatar->get_smart_avatar_url(
				$user_name, $user_email, $user_url, '');
			$disabled = ' readonly="readonly"';
		}
		if ($this->m_show_page_part & COMVATAR_MANAGER_SHOW_DISPLAY_AVATAR)
		{
?>
	<form name="display_avatar" method="post" action="<?php echo $this->m_manager_page_url; ?>">
		<input type="hidden" name="action" value="change_avatar_feedback" />
		<input type="hidden" name="email" value="<?php echo $_REQUEST['email']; ?>" />
		<input type="hidden" name="key" value="<?php echo $this->create_key($_REQUEST['email'], 'change_avatar_feedback'); ?>" />
		<input type="hidden" name="avatar" value="" />
<?php if ($this->m_comment_comvatar_url === false) : ?>
		<p>
			<?php printf(__('Currently you have not written any comments. The site system would use the following avatar for you.', COMVATAR_PLUGIN_ID)); ?>

		</p>
<?php elseif (empty($this->m_comment_comvatar_url)) : ?>
		<p>
			<?php printf(__('Currently you have not selected an avatar. The site system will use the following avatar for you.', COMVATAR_PLUGIN_ID)); ?>

		</p>
<?php else : ?>
		<p>
			<?php printf(__("You are currently using the follwing avatar. You can reset this to the site default by pressing on the button at the bottom of the 'Select Avatar' box below. You will be required to confirm this request by clicking a special link sent to your current e-mail address.", COMVATAR_PLUGIN_ID)); ?>

		</p>
<?php endif; ?>
		<table width="100%" cellspacing="2" cellpadding="5" class="editform">
			<tr valign="top">
			<th width="33%" scope="row"><?php _e('Avatar address:', COMVATAR_PLUGIN_ID); ?></th>
				<td>
					<input name="current_avatar" type="text" id="current_avatar" size="80" value="<?php echo $avatar; ?>" readonly="readonly" />
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row"><?php _e('Avatar image:', COMVATAR_PLUGIN_ID); ?></th>
				<td>
					<img src="<?php echo $avatar; ?>" />
				</td>
			</tr>
		</table>
	</form>
<?php
		}
	}

	function show_change_avatar()
	{
		global $g_comvatar;
		$first = '?';
		if (strpos($this->m_manager_page_url, '?'))
			$first = '&amp;';
		$link = $this->m_manager_page_url.
			$first. 'action=change_avatar_feedback'.
			'&amp;email='. $_REQUEST['email'].
			'&amp;key='. $this->create_key($_REQUEST['email'], 'change_avatar_feedback').
			'&amp;avatar=';
		if ($this->m_show_page_part & COMVATAR_MANAGER_SHOW_CHANGE_AVATAR)
		{
			$limit = $g_comvatar->m_unregistered_avatar_limit;
			if (!empty($this->m_comment_comvatar_url))
				$limit -= 1;
?>
	<fieldset class="options">
		<legend><?php _e('Select Avatar', COMVATAR_PLUGIN_ID); ?></legend>
		<form name="change_email" method="post" action="<?php echo $this->m_manager_page_url; ?>">
			<input type="hidden" name="action" value="change_avatar_feedback" />
			<input type="hidden" name="email" value="<?php echo $_REQUEST['email']; ?>" />
			<input type="hidden" name="key" value="<?php echo $this->create_key($_REQUEST['email'], 'change_avatar_feedback'); ?>" />
			<input type="hidden" name="avatar" value="" />
<?php if ($g_comvatar->get_num_of_unregistered($g_comvatar->m_registered_pool_files) <= $limit) : ?>
			<p>
				<?php _e('Sorry but no more avatars are currently available for registration. You may ask the site administrator to add some more avatars to the pool.', COMVATAR_PLUGIN_ID); ?>

			</p>
<?php else : ?>
			<p>
				<?php _e('The following avatars are available. If you already have selected an avatar, then the current avatar will be marked. Click on the preferred avatar to make this your new favorite. You will be required to confirm this request by clicking a special link sent to your current e-mail address.', COMVATAR_PLUGIN_ID); ?>

			</p>
<?php endif; ?>
			<ul class="<?php echo COMVATAR_CSS_CLASS; ?>-browse">
<?php foreach($g_comvatar->m_registered_pool_files as $file) { ?>
<?php if (!isset($li_class) || empty($li_class)) $li_class = 'alternate'; else $li_class = ''; ?>
<?php if ($this->is_selected_as_avatar($file['m_url'])) : ?>
				<li class="<?php echo $li_class; ?>">
					<span class="edit active"><img src="<?php echo $file['m_url']; ?>" /><br /><?php _e('Active', COMVATAR_PLUGIN_ID); ?></span>
				</li>
<?php elseif ($file['m_registered'] && $g_comvatar->m_display_registered) : ?>
<?php $owner_text = sprintf(__('Registered by %s', COMVATAR_PLUGIN_ID), $file['m_registered']); ?>
				<li class="<?php echo $li_class; ?>">
					<span class="edit used"><img src="<?php echo $file['m_url']; ?>" <?php if ($g_comvatar->m_display_owner) : ?> alt="<?php echo $owner_text; ?>" title="<?php echo $owner_text; ?>" <?php endif; ?>/><br /><?php _e('Used', COMVATAR_PLUGIN_ID); ?></span>
				</li>
<?php elseif (!$file['m_registered']) : ?>
				<li class="<?php echo $li_class; ?>">
					<a class="edit" href="<?php echo $link. $file['m_url']; ?>"><img src="<?php echo $file['m_url']; ?>" /><br /><?php _e('Select', COMVATAR_PLUGIN_ID); ?></a>
				</li>
<?php endif; ?>
<?php } ?>
			</ul>
<?php if (!empty($this->m_comment_comvatar_url)) : ?>
			<p class="submit" style="clear:both">
				<input type="submit" value="<?php _e('Reset to site default &raquo;', COMVATAR_PLUGIN_ID); ?>" />
			</p>
<?php endif; ?>
		</form>
	</fieldset>
<?php
		}
	}

	function show_page()
	{
		global $g_comvatar;
		$this->execute_action();
		if ($this->m_show_page_part & COMVATAR_MANAGER_SHOW_DISPLAY_HEADER)
		{
?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html>
<head>
	<title><?php bloginfo('name'); ?> &rsaquo; <?php _e("Comvatars Manager", COMVATAR_PLUGIN_ID); ?></title>
	<link rel="stylesheet" href="<?php echo get_settings('siteurl'); ?>/wp-admin/wp-admin.css" type="text/css" />
	<meta http-equiv="Content-Type" content="text/plain;charset=<?php bloginfo('charset'); ?>" />
<?php $g_comvatar->add_admin_css_style(); ?>

</head>
<body>
<div id="wphead">
	<h1><?php echo wptexturize(get_settings(('blogname'))); ?> <span>(<a href="<?php echo get_settings('home') . '/'; ?>"><?php _e('View site') ?> &raquo;</a>)</span></h1>
</div>
<?php
		}
		$this->show_message();
		$this->show_search_registrations();
		if ($this->m_show_page_part & COMVATAR_MANAGER_SHOW_DISPLAY_AVATAR)
		{
?>
<div class="wrap">
	<h2><?php printf(__('Avatar Manager for %s', COMVATAR_PLUGIN_ID), $_REQUEST['email']); ?></h2>
<?php
			$this->show_display_avatar();
			$this->show_change_avatar();
?>
</div>
<?php
		}
		if ($this->m_show_page_part & COMVATAR_MANAGER_SHOW_DISPLAY_HEADER)
		{
?>
</body>
</html>
<?php
		}
	}
};

// --------------------------------------------------------------------------
// The Comvatar class, the runtime master brain of this plugin
// --------------------------------------------------------------------------

class Comvatar
{
	// static options
	var $m_version = COMVATAR_VERSION;
	var $m_base_path = 'wp-content/comvatars/';

	// persistent options
	var $m_favatar_redirection = true;
	var $m_anonymous_pool_path = '';
	var $m_identified_pool_path = '';
	var $m_registered_pool_path = '';
	var $m_use_default = true;
	var $m_default_icon_size = COMVATAR_ADMIN_ICON_SIZE;
	var $m_use_fixed_icon_size = true;
	var $m_registration_manager_url = '';
	var $m_display_registered = false;
	var $m_display_owner = false;
	var $m_frequent_author_limit = COMVATAR_DEFAULT_FREQUENT_AUTHOR_LIMIT;
	var $m_unregistered_avatar_limit = COMVATAR_DEFAULT_UNREGISTERED_AVATAR_LIMIT;
	var $m_admin_user_level = COMVATAR_DEFAULT_USER_LEVEL;

	// runtime options
	var $m_session_id = 0;
	var $m_registration_link_shown = false;
	var $m_favatar_plugin_installed = false;
	var $m_gravatar_plugin_installed = false;
	var $m_anonymous_pool_files;
	var $m_identified_pool_files;
	var $m_registered_pool_files;
	var $m_anonymous_name = '';
	var $m_snoopy;

	// ------------------------------------------------------------------------
	// initialization
	// ------------------------------------------------------------------------

	function Comvatar()
	{
		load_plugin_textdomain(COMVATAR_PLUGIN_ID);
		$this->m_anonymous_name = __('Anonymous', COMVATAR_PLUGIN_ID);
		$this->session_id = microtime();

		// set persistent options to default before trying to store them in the database
		$this->m_anonymous_pool_path = $this->m_base_path. 'anonymous-pool/';
		$this->m_identified_pool_path = $this->m_base_path. 'identified-pool/';
		$this->m_registered_pool_path = $this->m_base_path. 'registered-pool/';
		$this->m_registration_manager_url =
			get_settings('siteurl') . '/'. COMVATAR_MANAGER_PAGE;
		$options = get_option(COMVATAR_PLUGIN_ID);
		if ($options === FALSE)
			$this->install_plugin();
		else
			$this->upgrade_plugin($options['version']);
		$this->load_options();
		$this->init_pool($this->m_anonymous_pool_files,
			$this->m_anonymous_pool_path);
		$this->init_pool($this->m_identified_pool_files,
			$this->m_identified_pool_path);
		$this->init_pool($this->m_registered_pool_files,
			$this->m_registered_pool_path);

		// find out whether depending base plugins are installed and activated; code
		// borrowed from Plugins Used; Andrew you'll get it back after we finished here ;)
		$activePluginsResult = get_settings('active_plugins');
		if (is_array($activePluginsResult))
			$pup_plugin_files = $activePluginsResult;
		else
			$pup_plugin_files = explode("\n",$activePluginsResult);
		$myPluginFiles = array_values($pup_plugin_files);
		if (is_array($myPluginFiles[0])) {
			$myPluginFiles = array_keys($pup_plugin_files);
		}
		$myPluginFiles = array_unique($myPluginFiles);
		foreach($myPluginFiles as $plugin_file)
		{
			if (trim($plugin_file) == "")
				continue;
			else if (trim($plugin_file) == "gravatars.php")
			{
				global $grav_options;
				if (!isset($grav_options))
					continue;
				$this->m_gravatar_plugin_installed = true;
				continue;
			}
			else if (trim($plugin_file) == "favatars.php")
			{
				$this->m_favatar_plugin_installed = true;
				continue;
			}
		}

		// this installs redirection support for favatar detection
		if ($this->m_favatar_plugin_installed && $this->m_favatar_redirection)
		{
			require_once(ABSPATH. 'wp-includes/class-snoopy.php');
			$this->m_snoopy = new Snoopy();
			remove_action('pingback_post', 'discover_favicon');
			remove_action('trackback_post', 'discover_favicon');
			remove_action('comment_post', 'discover_favicon');
			add_action('comment_post', 'update_comment_favicon');
			add_action('trackback_post', 'update_comment_favicon');
			add_action('pingback_post', 'update_comment_favicon');
		}

		// fix gravatar options
		if ($this->m_gravatar_plugin_installed)
		{
			global $grav_options;
			gravatar_defaults('update', $grav_options['gravatar_rating'],
				$grav_options['gravatar_size'], $grav_options['gravatar_border'],
				$grav_options['gravatar_default'], 0,
				$grav_options['gravatar_in_posts'], 0, 0);
		}

		// install callbacks
		add_filter('the_content', 'replace_html_comvatar_tag');
		add_filter('get_comment_author_link', 'get_default_class_image');
		add_filter('comment_author', 'get_default_class_image');
		add_action('comment_form', 'show_registration_manager_link');
		add_action('admin_menu', 'add_admin_page');
		add_action('admin_head', 'add_admin_css_style');
		add_action('wp_head', 'add_wp_css_style');
	}

	function install_plugin()
	{
		// plugin is newly installed
		global $wpdb;
		$query = "
			ALTER TABLE `$wpdb->comments`
			ADD COLUMN `comment_comvatar_url` TEXT
			NOT NULL DEFAULT ''
		";
		@$wpdb->query($query);
		$this->save_options();
	}

	function upgrade_plugin($old_version)
	{
		while ($old_version != COMVATAR_VERSION)
		{
			$this->load_options();
			if ($old_version == '2.0')
			{
				$old_version = '2.1';
			}
			else if ($old_version == '2.1')
			{
				$old_version = '2.2';
			}
			else if ($old_version == '2.2')
			{
				$old_version = '2.3';
			}
			else
			{
				die(sprintf(__('No upgrade path. This is version %s of the plugin, previously version %s was.', COMVATAR_PLUGIN_ID),
					COMVATAR_VERSION, $old_verion));
			}
			$this->version = $old_version;
			$this->save_options();
			$this->version = COMVATAR_VERSION;
		}
	}

	function fix_options()
	{
		$this->m_anonymous_pool_path = trim($this->m_anonymous_pool_path);
		if (substr($this->m_anonymous_pool_path, 0, 1) == '\\'
			|| substr($this->m_anonymous_pool_path, 0, 1) == '/')
			$this->m_anonymous_pool_path = substr($this->m_anonymous_pool_path, 1);

		$this->m_identified_pool_path = trim($this->m_identified_pool_path);
		if (substr($this->m_identified_pool_path, 0, 1) == '\\'
			|| substr($this->m_identified_pool_path, 0, 1) == '/')
			$this->m_identified_pool_path = substr($this->m_identified_pool_path, 1);

		$this->m_registered_pool_path = trim($this->m_registered_pool_path);
		if (substr($this->m_registered_pool_path, 0, 1) == '\\'
			|| substr($this->m_registered_pool_path, 0, 1) == '/')
			$this->m_registered_pool_path = substr($this->m_registered_pool_path, 1);

		if (!is_numeric($this->m_default_icon_size))
			$this->m_default_icon_size = COMVATAR_ADMIN_ICON_SIZE;
		else if ($this->m_default_icon_size < 0)
			$this->m_default_icon_size = 0;

		$this->m_registration_manager_url = trim($this->m_registration_manager_url);

		if (!is_numeric($this->m_frequent_author_limit))
			$this->m_frequent_author_limit = COMVATAR_DEFAULT_FREQUENT_AUTHOR_LIMIT;
		else if ($this->m_frequent_author_limit < 1)
			$this->m_frequent_author_limit = 1;

		if (!is_numeric($this->m_unregistered_avatar_limit))
			$this->m_unregistered_avatar_limit = COMVATAR_DEFAULT_UNREGISTERED_AVATAR_LIMIT;
		else if ($this->m_unregistered_avatar_limit < 0)
			$this->m_unregistered_avatar_limit = 0;

		if (!is_numeric($this->m_admin_user_level))
			$this->m_admin_user_level = COMVATAR_DEFAULT_USER_LEVEL;
		else if ($this->m_admin_user_level < 0)
			$this->m_admin_user_level = 0;
		else if ($this->m_admin_user_level > 10)
			$this->m_admin_user_level = 10;
	}

	function save_options()
	{
		$this->fix_options();
		$options = array(
			'version' => $this->m_version,
			'favatar_redirection' => $this->m_favatar_redirection,
			'anonymous_pool_path' => $this->m_anonymous_pool_path,
			'identified_pool_path' => $this->m_identified_pool_path,
			'registered_pool_path' => $this->m_registered_pool_path,
			'use_default' => $this->m_use_default,
			'default_icon_size' => $this->m_default_icon_size,
			'use_fixed_icon_size' => $this->m_use_fixed_icon_size,
			'registration_manager_url' => $this->m_registration_manager_url,
			'display_registered' => $this->m_display_registered,
			'display_owner' => $this->m_display_owner,
			'frequent_author_limit' => $this->m_frequent_author_limit,
			'unregistered_avatar_limit' => $this->m_unregistered_avatar_limit,
			'admin_user_level' => $this->m_admin_user_level
		);
		update_option(COMVATAR_PLUGIN_ID, $options);
	}

	function load_options()
	{
		$options = get_option(COMVATAR_PLUGIN_ID);
		$this->m_favatar_redirection = $options['favatar_redirection'];
		$this->m_anonymous_pool_path = $options['anonymous_pool_path'];
		$this->m_identified_pool_path = $options['identified_pool_path'];
		// introduced in version 2.1
		if (isset($options['registered_pool_path']))
			$this->m_registered_pool_path = $options['registered_pool_path'];
		else
			$this->m_registered_pool_path = $this->m_identified_pool_path;

		$this->m_use_default = $options['use_default'];
		// introduced in version 2.3
		if (isset($options['default_icon_size']))
			$this->m_default_icon_size = $options['default_icon_size'];
		else
			$this->m_default_icon_size = COMVATAR_ADMIN_ICON_SIZE;
		if (isset($options['use_fixed_icon_size']))
			$this->m_use_fixed_icon_size = $options['use_fixed_icon_size'];
		else
			$this->m_use_fixed_icon_size = true;

		$this->m_registration_manager_url = $options['registration_manager_url'];
		$this->m_display_registered = $options['display_registered'];
		$this->m_display_owner = $options['display_owner'];
		$this->m_frequent_author_limit = $options['frequent_author_limit'];
		$this->m_unregistered_avatar_limit = $options['unregistered_avatar_limit'];
		$this->m_admin_user_level = $options['admin_user_level'];
		$this->fix_options();
	}

	// ------------------------------------------------------------------------
	// credential detection
	// ------------------------------------------------------------------------

	function create_key($email, $action)
	{
		return md5($email. '|'. $action. '|'. COMVATAR_PLUGIN_ID. '|'. DB_PASSWORD);
	}

	function get_comment_author_url()
	{
		global $wpdb, $comment;

		if (isset($comment) && isset($comment->comment_ID)
			&& is_numeric($comment->comment_ID))
		{
			$query = "
				SELECT `comment_author_url`
				FROM `$wpdb->comments`
				WHERE `comment_ID` = '$comment->comment_ID'
			";
			$url = @trim(@$wpdb->get_var($query));
		}
		if (!isset($url))
			$url = '';
		return $url;
	}

	function get_comment_author_email()
	{
		global $wpdb, $comment;

		if (isset($comment) && isset($comment->comment_ID)
			&& is_numeric($comment->comment_ID))
		{
			$query = "
				SELECT `comment_author_email`
				FROM `$wpdb->comments`
				WHERE `comment_ID` = '$comment->comment_ID'
			";
			$email = @trim(@$wpdb->get_var($query));
		}
		if (!isset($email))
			$email = '';
		return $email;
	}

	function get_comment_author_name()
	{
		global $wpdb, $comment;

		if (isset($comment) && isset($comment->comment_ID)
			&& is_numeric($comment->comment_ID))
		{
			$query = "
				SELECT `comment_author`
				FROM `$wpdb->comments`
				WHERE `comment_ID` = '$comment->comment_ID'
			";
			$name = @trim(@$wpdb->get_var($query));
		}
		if (!isset($name))
			$name = '';
		return $name;
	}

	function get_string_type($string='', &$name, &$email, &$url)
	{
		$name = '';
		$email = '';
		$url = '';

		if (is_array($string))
		{
			if (isset($string[0]))
				$name = @trim($string[0]);
			if (isset($string[1]))
				$email = @trim($string[1]);
			if (isset($string[2]))
				$url = @trim($string[2]);
			return 3;
		}

		$string = @trim($string);
		if (empty($string))
		{
			$name = $this->get_comment_author_name();
			$email = $this->get_comment_author_email();
			$url = $this->get_comment_author_url();
			return 4;
		}

		if (is_email($string))
		{
			$email = $string;
			return 1;
		}
		else if (($r = strpos(strtolower($string), "http://")) !== false && $r == 0)
		{
			$url = $string;
			return 2;
		}
		$name = $string;
		return 0;
	}

	// ------------------------------------------------------------------------
	// comvatar registration
	// ------------------------------------------------------------------------

	function fix_registered_urls($old_registered_pool_path)
	{
		// this function will change all comvatar urls in the database that are
		// stored in the registrated_pool on this server to the new url after you
		// have changed the path to registrated_pool.
		global $wpdb;

		if ($old_registered_pool_path != $this->m_registered_pool_path)
		{
			$query = "
				SELECT `comment_author_email`, `comment_comvatar_url`
				FROM `$wpdb->comments`
				WHERE `comment_comvatar_url` <> ''
				GROUP BY `comment_author_email`, `comment_comvatar_url`
				ORDER BY `comment_author_email`, `comment_comvatar_url`
			";
			$result = $wpdb->get_results($query);
			if (!empty($result))
			{
				foreach ($result as $item)
				{
					if (!strncmp($item->comment_comvatar_url, get_settings('siteurl').'/',
						strlen(get_settings('siteurl'))+1))
					{
						// hopefully the path is only one time part of the url
						// otherwise we may fail here
						$url = str_replace($old_registered_pool_path,
							$this->m_registered_pool_path, $item->comment_comvatar_url);
						$query = "
							UPDATE `$wpdb->comments`
							SET `comment_comvatar_url` = '$url'
							WHERE comment_author_email = '$item->comment_author_email'
							AND comment_comvatar_url = '$item->comment_comvatar_url'
						";
						$wpdb->query($query);
					}
				}
			}
		}
	}

	function init_pool(&$pool_files, $pool_path)
	{
		global $wpdb;

		$pool_files = array();
		$pool_dir = @dir(ABSPATH. $pool_path);
		if ($pool_dir)
		{
			$query = "
				SELECT DISTINCT `comment_comvatar_url`,
					`comment_author_email`
				FROM `$wpdb->comments`
				WHERE TRIM(`comment_comvatar_url`) <> ''
				ORDER BY `comment_comvatar_url`
			";
			$result = $wpdb->get_results($query);

			$file_name = $pool_dir->read();
			while($file_name !== false)
			{
				$file_url = get_settings('siteurl').'/'.$pool_path. $file_name;
				if ($file_name !=	'..' && $file_name !=	'.'
					&& is_file(ABSPATH. $pool_path. $file_name)
					&& is_readable(ABSPATH. $pool_path. $file_name))
				{
					$registered_by = false;
					if (!empty($result))
					{
						foreach ($result as $item)
						{
							if ($file_url == $item->comment_comvatar_url)
							{
								$registered_by = $item->comment_author_email;
								break;
							}
						}
					}

					$pool_files[] = array('m_url' => $file_url,
						'm_registered' => $registered_by);
				}
				$file_name = $pool_dir->read();
			}
		}
		sort($pool_files);
	}

	function get_current_user_email()
	{
		global $wpdb, $user_email;

		get_currentuserinfo();
		$account_email = trim($user_email);
		if (is_email($account_email))
			return $account_email;

		if (isset($_COOKIE['comment_author_email_'. COOKIEHASH]))
		{
			$author_email = trim($_COOKIE['comment_author_email_'. COOKIEHASH]);
			if (is_email($author_email))
				return $author_email;
		}
		return '';
	}

	function get_num_of_comments($user_email)
	{
		global $wpdb;

		$user_email = trim($user_email);
		if (!is_email($user_email))
			return -1;
		$query = "
			SELECT COUNT(*)
			FROM `$wpdb->comments`
			WHERE LCASE(`comment_author_email`) = '$user_email'
		";
		return $wpdb->get_var($query);
	}

	function get_num_of_unregistered($pool_files)
	{
		$n = 0;
		foreach ($pool_files as $file)
		{
			if (!$file['m_registered'])
				++$n;
		}
		return $n;
	}

	function get_registered_comvatar_url($user_email)
	{
		global $wpdb;

		$user_email = trim($user_email);
		if (!is_email($user_email))
			return '';
		$query = "
			SELECT `comment_comvatar_url`
			FROM `$wpdb->comments`
			WHERE LCASE(`comment_author_email`) = '$user_email'
			LIMIT 1
		";
		return $wpdb->get_var($query);
	}

	function get_registration_manager_url($user_email, $html_encode=true,
		$display=false)
	{
		if ($html_encode)
			$amp = 'amp;';

		$url = $this->m_registration_manager_url.
			'?action=display_avatar'.
			'&'. $amp. 'email='. $user_email.
			'&'. $amp. 'key='. $this->create_key($user_email, 'display_avatar');

		if (!$display)
			return $url;
		echo $url;
	}

	function get_registration_manager_link($user_email, $display=false)
	{
		if ($this->get_num_of_comments($user_email) >= $this->m_frequent_author_limit)
		{
			$text = sprintf(__('As a frequent comment author you are allowed to <a href="%s">register your own avatar</a>.',
				COMVATAR_PLUGIN_ID), $this->get_registration_manager_url($user_email));
			$html = sprintf('<p style="clear: both;" class="%s-registration-link">%s</p>',
				COMVATAR_CSS_CLASS, $text);
		}

		if (!$display)
			return $html;
		echo $html;
	}

	function show_registration_manager_link($id='0')
	{
		if ($this->m_registration_link_shown)
			return $id;
		$user_email = $this->get_current_user_email();
		$this->get_registration_manager_link($user_email, true);
		$this->m_registration_link_shown = true;
		return $id;
	}

	// ------------------------------------------------------------------------
	// comvatar random
	// ------------------------------------------------------------------------

	function get_comvatar_from_pool($pool_files, $user_name, $user_email,
		$default_avatar_url='')
	{
		$default_avatar_url = trim($default_avatar_url);
		if (empty($pool_files) || !empty($default_avatar_url))
			return @trim($default_avatar_url);
		$hash = crc32($user_name.'|'.$user_email);
		$index = abs($hash % count($pool_files));

		// skip registered avatars
		$n = $index;
		while ($pool_files[$n]['m_registered'] && $n > 0)
			--$n;
		if ($pool_files[$n]['m_registered'])
			$n = count($pool_files)-1;
		while ($pool_files[$n]['m_registered'] && $n > $index)
			--$n;
		return $pool_files[$n]['m_url'];
	}

	function get_comvatar_url($user_name='', $user_email='',
		$default_avatar_url='')
	{

		$user_name = @trim(strtolower($user_name));
		if (empty($user_name))
			$user_name = $this->m_session_id;

		$user_email = @trim(strtolower($user_email));
		if (empty($user_email))
			$user_email = COMVATAR_ANONYMOUS_EMAIL;

		if ($user_email == COMVATAR_ANONYMOUS_EMAIL)
			$avatar_url = $this->get_comvatar_from_pool($this->m_anonymous_pool_files,
				$user_name, $user_email, $default_avatar_url);
		else if (is_dir(ABSPATH. $this->m_identified_pool_path))
			$avatar_url = $this->get_comvatar_from_pool($this->m_identified_pool_files,
				$user_name, $user_email, $default_avatar_url);
		else
			$avatar_url = @trim($default_avatar_url);
		return $avatar_url;
	}

	// ------------------------------------------------------------------------
	// favatar functionality
	// ------------------------------------------------------------------------

	function fix_url(&$url)
	{
		$url = trim($url);
		if (!empty($url) && (($r = strpos(strtolower($url), "http://")) === false
			|| $r > 0))
			$url = 'http://'.$url;
	}

	function update_comment_favicon($comment_ID)
	{
		// use WP built in snoopy class to find out the the favicon. By that
		// also URL redirection can be handled
		global $wpdb;

		if ($this->m_favatar_plugin_installed)
		{
			if ($this->m_favatar_redirection)
			{
				$url = $wpdb->get_var("
					SELECT comment_author_url
					FROM `$wpdb->comments`
					WHERE `comment_ID` = '$comment_ID'
				");
				if (!isset($url) || empty($url) || !$this->m_snoopy->fetch($url))
					return $comment_ID;
				$redirected_url = $this->m_snoopy->lastredirectaddr;
				if (!isset($redirected_url) || empty($redirected_url))
					$comment_ID = discover_favicon($comment_ID);
				else
				{
					$wpdb->query("
						UPDATE `$wpdb->comments`
						SET `comment_author_url` = '$redirected_url'
						WHERE `comment_ID` = '$comment_ID'
					");
					$comment_ID = discover_favicon($comment_ID);
					$wpdb->query("
						UPDATE `$wpdb->comments`
						SET `comment_author_url` = '$url'
						WHERE `comment_ID` = '$comment_ID'
					");
				}
			}
			else
				$comment_ID = discover_favicon($comment_ID);
		}
		return $comment_ID;
	}

	function get_favatar_url($user_url, $default_avatar_url='')
	{
		global $wpdb, $comment;

		if ($this->m_favatar_plugin_installed)
		{
			$this->fix_url($user_url);
			if (isset($comment) && isset($comment->comment_ID)
				&& is_numeric($comment->comment_ID))
			{
				$avatar_url = @trim(@$wpdb->get_var("
					SELECT `comment_favicon_url`
					FROM `$wpdb->comments`
					WHERE `comment_ID` = '$comment->comment_ID'
					AND `comment_author_url` = '$user_url'
				"));
			}
			else if (isset($user_url) && !empty($user_url))
			{
				if ($this->m_favatar_redirection && $this->m_snoopy->fetch($user_url))
					$user_url = $this->m_snoopy->lastredirectaddr;
				$avatar_url = getFavicon($user_url);
			}
		}
		if (!isset($avatar_url) || empty($avatar_url))
			$avatar_url = @trim($default_avatar_url);
		return $avatar_url;
	}

	// ------------------------------------------------------------------------
	// gravatar functionality
	// ------------------------------------------------------------------------

	function get_gravatar_url($user_email='', $icon_size, $default_avatar_url='')
	{
		$user_email = @trim(strtolower($user_email));
		if (empty($user_email))
			$user_email = COMVATAR_ANONYMOUS_EMAIL;

		$icon_size = trim($icon_size);
		if ($this->m_gravatar_plugin_installed)
		{
			global $grav_options;
			if (is_numeric($icon_size))
			{
				$saved_gravatar_size = $grav_options['gravatar_size'];
				$grav_options['gravatar_size'] = $icon_size;
			}
			$avatar_url = wp_gravatar($user_email, $default_avatar_url);
			if (is_numeric($icon_size))
				$grav_options['gravatar_size'] = $saved_gravatar_size;
		}
		// in case gravatar.com is unavailable
		if (!isset($avatar_url) || !$avatar_url || empty($avatar_url))
			$avatar_url = @trim($default_avatar_url);
		return $avatar_url;
	}

	// ------------------------------------------------------------------------
	// assembly functions
	// ------------------------------------------------------------------------

	function get_smart_avatar_url($user_name, $user_email,
		$user_url, $icon_size, $default_avatar_url='')
	{
		$registered_avatar_url = $this->get_registered_comvatar_url($user_email);
		if (!empty($registered_avatar_url))
			return $registered_avatar_url;
		$default_avatar_url = $this->get_comvatar_url($user_name, $user_email, $default_avatar_url);
		$default_avatar_url = $this->get_favatar_url($user_url, $default_avatar_url);
		return $this->get_gravatar_url($user_email, $icon_size, $default_avatar_url);
	}

  function get_smart_avatar_image(&$user_name, &$user_email,
		&$user_url, $icon_size, $class_name, $default_avatar_url='')
	{
		$avatar_url = $this->get_smart_avatar_url($user_name, $user_email,
			$user_url, $icon_size, $default_avatar_url);
		$html = '<img class="'. $class_name.
			'" src="'. $avatar_url.
			'" alt="'. htmlentities($user_name, ENT_QUOTES, get_settings('blog_charset'));
		$html .= '" />';
	  return $html;
	}

  function get_smart_avatar_image_link(&$user_name, &$user_email,
		&$user_url, $icon_size, $class_name, $default_avatar_url='')
	{
		$this->fix_url($user_url);
		$avatar_image = $this->get_smart_avatar_image($user_name, $user_email,
			$user_url, $icon_size, $class_name, $default_avatar_url);
		$html = '';
		if (!empty($user_url))
			$html .= '<a class="'. $class_name. '" href="'. $user_url. '">';
		$html .= $avatar_image;
		if (!empty($user_url))
			$html .= '</a>';
		return $html;
	}

	// ------------------------------------------------------------------------
	// admin pages
	// ------------------------------------------------------------------------

	function management_page()
	{
		require_once(ABSPATH. COMVATAR_MANAGER_PAGE);
	}

	function options_page()
	{
		if($_SERVER['REQUEST_METHOD'] == 'POST')
		{
			if (isset($_POST['comvatars_options']) && ($_POST['comvatars_options'] == 'update'))
			{
				// update the comvatar options
				$old_registered_pool_path = $this->m_registered_pool_path;
				$this->m_favatar_redirection = isset($_POST['favatar_redirection']);
				$this->m_anonymous_pool_path = $_POST['anonymous_pool_path'];
				$this->m_identified_pool_path = $_POST['identified_pool_path'];
				$this->m_registered_pool_path = $_POST['registered_pool_path'];

				$this->m_use_default = isset($_POST['use_default']);
				$this->m_default_icon_size = $_POST['default_icon_size'];
				$this->m_use_fixed_icon_size = isset($_POST['use_fixed_icon_size']);

				$this->m_registration_manager_url = $_POST['registration_manager_url'];
				$this->m_display_registered = isset($_POST['display_registered']);
				$this->m_display_owner = isset($_POST['display_owner']);
				$this->m_frequent_author_limit = $_POST['frequent_author_limit'];
				$this->m_unregistered_avatar_limit = $_POST['unregistered_avatar_limit'];
				$this->m_admin_user_level = $_POST['admin_user_level'];

				$this->save_options();
				$this->fix_registered_urls($old_registered_pool_path);
				$this->init_pool($this->m_anonymous_pool_files, $this->m_anonymous_pool_path);
				$this->init_pool($this->m_identified_pool_files, $this->m_identified_pool_path);
				$this->init_pool($this->m_registered_pool_files, $this->m_registered_pool_path);
				echo "<div class='updated'><p><strong>". __('Options saved.'). "</strong></p></div>\n";
			}
		}

		// check loaded parameter
		if (empty($this->m_anonymous_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for anonymous user is not set.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!file_exists(ABSPATH. $this->m_anonymous_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for anonymous user does not exist.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!is_readable(ABSPATH. $this->m_anonymous_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for anonymous user is not readable.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!is_dir(ABSPATH. $this->m_anonymous_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Path to avatars for anonymous user does not point to a directory.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (empty($this->m_anonymous_pool_files))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				 __('Directory for avatars for anonymous user does not contain any usable images.', COMVATAR_PLUGIN_ID).
				 "</strong></p></div>\n";

		if (empty($this->m_identified_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for identified user is not set.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!file_exists(ABSPATH. $this->m_identified_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for identified user does not exist.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!is_readable(ABSPATH. $this->m_identified_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for identified user is not readable.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!is_dir(ABSPATH. $this->m_identified_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Path to avatars for identified user does not point to a directory.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (empty($this->m_identified_pool_files))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for identified user does not contain any usable images.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";

		if (empty($this->m_registered_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for registered user is not set.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!file_exists(ABSPATH. $this->m_registered_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for registered user does not exist.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!is_readable(ABSPATH. $this->m_registered_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for registered user is not readable.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (!is_dir(ABSPATH. $this->m_registered_pool_path))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Path to avatars for registered user does not point to a directory.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (empty($this->m_registered_pool_files))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('Directory for avatars for registered user does not contain any usable images.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";

		if (empty($this->m_registration_manager_url))
			echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
				__('URL to registration manager is not set.', COMVATAR_PLUGIN_ID).
				"</strong></p></div>\n";
		else if (ini_get('allow_url_fopen'))
		{
			if (($file = @fopen($this->m_registration_manager_url, 'r')) === false)
				echo "<div class='". COMVATAR_CSS_CLASS. "-error'><p><strong>".
					__('URL to registration manager is not accessible.', COMVATAR_PLUGIN_ID).
					"</strong></p></div>\n";
			else
				fclose($file);
		}
?>
<div class="wrap">
	<h2><?php _e('Comvatars Options', COMVATAR_PLUGIN_ID) ?></h2>
	<form method="post">
		<input type='hidden' name='comvatars_options' value='update' />
		<table width="100%" cellspacing="2" cellpadding="5" class="editform">
			<tr valign="top">
				<th width="33%" scope="row"><?php _e('Favatar redirection:', COMVATAR_PLUGIN_ID); ?></th>
				<td>
					<label for="favatar_redirection">
						<input name="favatar_redirection" type="checkbox" id="favatar_redirection" value="true" <?php if ($this->m_favatar_redirection) : ?>checked="checked" <?php endif; ?>/>
						<?php _e('If activated favicons will also be found if the users URL is redirected.', COMVATAR_PLUGIN_ID); ?>

					</label>
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row"><?php _e('Path to avatars for anonymous user:', COMVATAR_PLUGIN_ID); ?></th>
				<td>
					<label for="anonymous_pool_path">
						<input name="anonymous_pool_path" type="text" id="anonymous_pool_path" value="<?php echo $this->m_anonymous_pool_path ?>" size="80" />
						<br /><?php printf(__('The directory path to the Comvatars image pool relative to your wordpress address &quot;%s&quot;.', COMVATAR_PLUGIN_ID), get_settings('siteurl').'/'); ?>

					</label>
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row"><?php _e('Path to avatars for identified user:', COMVATAR_PLUGIN_ID); ?></th>
				<td>
					<label for="identified_pool_path">
						<input name="identified_pool_path" type="text" id="identified_pool_path" value="<?php echo $this->m_identified_pool_path; ?>" size="80" />
						<br /><?php printf(__('The directory path to the Comvatars image pool relative to your wordpress address &quot;%s&quot;.', COMVATAR_PLUGIN_ID), get_settings('siteurl').'/'); ?>

					</label>
				</td>
			</tr>
			<tr valign="top">
				<th width="33%" scope="row"><?php _e('Path to avatars for registered user:', COMVATAR_PLUGIN_ID); ?></th>
				<td>
					<label for="registered_pool_path">
						<input name="registered_pool_path" type="text" id="registered_pool_path" value="<?php echo $this->m_registered_pool_path; ?>" size="80" />
						<br /><?php printf(__('The directory path to the Comvatars image pool relative to your wordpress address &quot;%s&quot;.', COMVATAR_PLUGIN_ID), get_settings('siteurl').'/'); ?>

					</label>
				</td>
			</tr>
		</table>

		<fieldset class="options">
			<legend>
				<input name="use_default" type="checkbox" id="use_default" value="true" <?php if ($this->m_use_default) : ?>checked="checked" <?php endif; ?>/>
				<label for="use_default"><?php _e('Use default Wordpress integration', COMVATAR_PLUGIN_ID); ?></label>
			</legend>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Use avatar fixed size:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<label for="use_fixed_icon_size">
							<input name="use_fixed_icon_size" type="checkbox" id="use_fixed_icon_size" value="true" <?php if ($this->m_use_fixed_icon_size) : ?>checked="checked" <?php endif; ?>/>
							<?php _e('Check this if you want to have all avatar images appear in the same size of pixel as set below instead of using their native size.', COMVATAR_PLUGIN_ID); ?>

						</label>
					</td>
				</tr>
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Avatar size:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<label for="default_icon_size">
							<input name="default_icon_size" type="text" id="default_icon_size" value="<?php echo $this->m_default_icon_size; ?>" size="2" />
							<br /><?php _e('The size of the avatar image in pixel used for all avatars.', COMVATAR_PLUGIN_ID); ?>

						</label>
					</td>
				</tr>
			</table>
		</fieldset>

		<fieldset class="options">
			<legend><?php _e('Avatar Registration Manager', COMVATAR_PLUGIN_ID) ?></legend>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Display registered images:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<label for="display_registered">
							<input name="display_registered" type="checkbox" id="display_registered" value="true" <?php if ($this->m_display_registered) : ?>checked="checked" <?php endif; ?>/>
							<?php _e('Also displays already registered avatars but will mark them as unavailable.', COMVATAR_PLUGIN_ID); ?>

						</label>
					</td>
				</tr>
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Display avatar owner:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<label for="display_owner">
							<input name="display_owner" type="checkbox" id="display_owner" value="true" <?php if ($this->m_display_owner) : ?>checked="checked" <?php endif; ?>/>
							<?php _e('For Admins displays the e-mail address of the avatar owner in the &quot;alt&quot; and &quot;title&quot; attribute of the image.', COMVATAR_PLUGIN_ID); ?>

						</label>
					</td>
				</tr>
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('URL to registration manager:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<label for="registration_manager_url">
							<input name="registration_manager_url" type="text" id="registration_manager_url" value="<?php echo $this->m_registration_manager_url; ?>" size="80" />
							<br /><?php printf(__('The URL to the registration manager site. Change this if you want to supply your own styled registration manager by using the appropriate template tag on your page.', COMVATAR_PLUGIN_ID), get_settings('siteurl').'/'); ?>

						</label>
					</td>
				</tr>
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Frequent comments limit:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<label for="frequent_author_limit">
							<input name="frequent_author_limit" type="text" id="frequent_author_limit" value="<?php echo $this->m_frequent_author_limit; ?>" size="2" />
							<br /><?php _e('The minimum number of comments by one user as from he will be allowed to declare his own avatar.', COMVATAR_PLUGIN_ID); ?>

						</label>
					</td>
				</tr>
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Unregistered image limit:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<label for="unregistered_avatar_limit">
							<input name="unregistered_avatar_limit" type="text" id="unregistered_avatar_limit" value="<?php echo $this->m_unregistered_avatar_limit; ?>" size="2" />
							<br /><?php _e('The minimum number of images in the identified pool that have to remain unregisterd.', COMVATAR_PLUGIN_ID); ?>

						</label>
					</td>
				</tr>
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Admin user level:', COMVATAR_PLUGIN_ID); ?></th>
					<td>
						<label for="admin_user_level">
							<input name="admin_user_level" type="text" id="admin_user_level" value="<?php echo $this->m_admin_user_level; ?>" size="2" />
							<br /><?php _e('The minimum user level a user needs to browse the avatar registration database.', COMVATAR_PLUGIN_ID); ?>

						</label>
					</td>
				</tr>
			</table>
		</fieldset>
		<p class="submit">
			<input type="submit" name="submit" value="<?php _e('Update Options') ?> &raquo;" />
		</p>
	</form>
</div>
<?php
	}

	// ------------------------------------------------------------------------
	// class filter functions
	// ------------------------------------------------------------------------

	function add_admin_page()
	{
		add_management_page(__('Comvatars Manager', COMVATAR_PLUGIN_ID),
			__('Comvatars', COMVATAR_PLUGIN_ID), 8, __FILE__,
			array($this, 'management_page'));
		add_options_page(__('Comvatars', COMVATAR_PLUGIN_ID),
			__('Comvatars', COMVATAR_PLUGIN_ID), 5, __FILE__,
				array($this, 'options_page'));
	}

	function add_admin_css_style()
	{
		$this->use_default_class_image = true;
?>
	<style type="text/css" media="screen">
	*.<?php echo COMVATAR_CSS_CLASS; ?>-admin {
		border: 0;
		text-decoration: none;
		width: <?php echo COMVATAR_ADMIN_ICON_SIZE; ?>px;
		height: <?php echo COMVATAR_ADMIN_ICON_SIZE; ?>px;
	}
	*.<?php echo COMVATAR_CSS_CLASS; ?>-error {
		background: #ffc1c1;
		border: 1px solid #c00000;
		margin: 1em 5% 10px;
		padding: 0 1em 0 1em;
	}
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li {
		float: left;
		list-style: none;
		margin: 1em;
		padding: 0;
		text-align: center;
	}
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li span.active {
		display: block;
		padding: 0;
		text-align: center;
		background: #bbeebb;
	}
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li span.used {
		display: block;
		padding: 0;
		text-align: center;
		background: #ffc1c1;
	}
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li a,
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li a:link,
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li a:visited,
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li a:hover,
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li a:focus,
	*.<?php echo COMVATAR_CSS_CLASS; ?>-browse li a:active {
		padding: 0;
	}
	</style>
<?php
		return true;
	}

	function add_wp_css_style()
	{
		if ($this->m_use_default && $this->m_use_fixed_icon_size)
		{
?>
	<style type="text/css" media="screen">
	img.<?php echo COMVATAR_CSS_CLASS; ?>-default {
		width: <?php echo $this->m_default_icon_size; ?>px;
		height: <?php echo $this->m_default_icon_size; ?>px;
	}
	</style>
<?php
		}
		return true;
	}

	function get_default_class_image($s)
	{
		if ($this->m_use_default || $this->use_default_class_image)
		{
			$css_class = '-default';
			$icon_size = false;
			if ($this->m_use_fixed_icon_size)
				$icon_size = $this->m_default_icon_size;
			if ($this->use_default_class_image)
			{
				// we are in admin mode
				$css_class = '-admin';
				$icon_size = COMVATAR_ADMIN_ICON_SIZE;
			}
			$this->get_string_type('', $user_name, $user_email, $user_url);
			$html = $this->get_smart_avatar_image_link($user_name, $user_email,
				$user_url, $icon_size,
				COMVATAR_CSS_CLASS. $css_class). ' '. $s;
			return $html;
		}
		return $s;
	}

	function replace_html_comvatar_tag_callback($match)
	{
		$name_index = 1;
		$value_index = 3;
		$pattern = "/[\s]*(.*?)[\s]*=[\s]*([\"])(.*?)\\2/is";
		preg_match_all($pattern, trim($match[2]), $elements);

		$class_name = COMVATAR_CSS_CLASS;

		$count = count($elements[0]);
		for ($n = 0; $n < $count; ++$n)
		{
			if (strtolower($elements[$name_index][$n]) == 'name')
				$this->get_string_type($elements[$value_index][$n], $user_name, $user_email, $user_url);
			else if (strtolower($elements[$name_index][$n]) == 'user')
				$user_name = $elements[$value_index][$n];
			else if (strtolower($elements[$name_index][$n]) == 'email')
				$user_email = $elements[$value_index][$n];
			else if (strtolower($elements[$name_index][$n]) == 'url')
				$user_url = $elements[$value_index][$n];
			else if (strtolower($elements[$name_index][$n]) == 'size')
				$icon_size = $elements[$value_index][$n];
			else if (strtolower($elements[$name_index][$n]) == 'default')
				$default_url = $elements[$value_index][$n];
			else if (strtolower($elements[$name_index][$n]) == 'class')
				$class_name = $elements[$value_index][$n];
		}
		if (empty($user_name) && empty($user_email) && empty($user_url))
			$this->get_string_type('', $user_name, $user_email, $user_url);

		return $this->get_smart_avatar_image_link($user_name, $user_email,
			$user_url, $icon_size, $class_name, $default_avatar_url);
	}

	function replace_html_comvatar_tag($content='')
	{
		/* Looks for all instances of <!--comvatar ...--> in the content
		and replaces it with an <a> and <img /> tag of the avatar. Only
		the <img /> tag will be returned if no link was provided or could
		be found.

		As with usual HTML tags you can define attributes for this tag
		in usual XML syntax, this is: attribute_name="attribute_value".
		If two attributes are mutual exclusive to each other, but are
		defined for the same tag, the last one specified is in charge.
		The following attributes are defined:

		name: If empty, or not defined and no user, email or url attribute
			is set, it will try to take the user information from the
			current comment. If no comment is processed at this time,
			the user name will be set to the localized text of Anonymous,
			the user email and user URL will be
			left blank. If name is set as string and is validated as,	URL
			the address will be taken to receive the favatar from this URL.
			If name	is set as string and is validated as email address, the
			address will be taken	to receive the gravatar from the Gravatar
			service. Otherwise the name will be used to select a Comvatar.
			This attribute is mutual exclusive to the attributes user, email, url.

		user: The name of the user. This will be used for Comvatars base functionality.
			This attribute is mutual exclusive to the attributes name.

		email: The email of a user. This will be used for Gravatars functionality.
			This attribute is mutual exclusive to the attributes name.

		url: The url of a user. This will be used for Favatars functionality.
			This attribute is mutual exclusive to the attributes name.

		size: The size of the returned Gravatar(!) image. This overrides
			the settings in the Gravatar admin panel. Because
			favicons, comvatars and gravatars can all be of different size
			no width or height attribute will be given to the <img /> tag.

		default: The default url to the avatar that should be displayed if all
			else fails. If set, the Comvatars functionality of selecting an
			avatar is overridden.

		class: (string) If set the default CSS class for th <img /> tag will
			overridden to the one defined in this parameter.
		*/

		$pattern = "/<!--[\s]*comvatar([\s]*(.*?)[\s]*)?-->/is";
		return preg_replace_callback($pattern,
			array($this, replace_html_comvatar_tag_callback), $content);
	}
}

// --------------------------------------------------------------------------
// internal filter function wrapper
// --------------------------------------------------------------------------

function update_comment_favicon($comment_ID)
{
	global $g_comvatar;
	return $g_comvatar->update_comment_favicon($comment_ID);
}

function replace_html_comvatar_tag($content)
{
	global $g_comvatar;
	return $g_comvatar->replace_html_comvatar_tag($content);
}

function get_default_class_image($s)
{
	global $g_comvatar;
	return $g_comvatar->get_default_class_image($s);
}

function add_admin_page()
{
	global $g_comvatar;
	return $g_comvatar->add_admin_page();
}

function add_admin_css_style()
{
	global $g_comvatar;
	return $g_comvatar->add_admin_css_style();
}

function add_wp_css_style()
{
	global $g_comvatar;
	return $g_comvatar->add_wp_css_style();
}

// --------------------------------------------------------------------------
// template tags
// --------------------------------------------------------------------------

function comvatar_url($name='', $default_avatar_url='', $icon_size='',
	$display=true)
{
	/* Returns an URL to an avatar image in all cases, either to a default one,
	favatar or gravatar.

	name: (mixed) If empty, it will try to take the user information from the
	current comment. If no comment is processed at this time, the user name
	will be set to the localized text of Anonymous, the user email
	and user URL will be left blank. If name is set as string and is validated as,
	URL the address will be taken to receive the favatar from this URL. If name
	is set as string and is validated as email address, the address will be taken
	to receive the gravatar from the Gravatar service. If name is an array, it
	is expected to contain the user name in element 0, the users email address
	in element 1 and the users URL in element 2. Otherwise the name will be used
	to select a Comvatar.

	default_avatar_url: (string) If set and no user name or user email is
	found valid or the script runs into an error this avatar will be displayed.
	The URL must be complete including 'http://'. This is not checked.
	If it is empty, the plugin chooses an default avatar from one of the pools
	image from the plugin folder.

	icon_size: (integer) If set and the Gravatars plugin is activated, it
	will override the Gravatars setting for this image and requests the image
	in the specified size from the gravatar service.

	display: (boolean) If true, the resulting output will be printed at the
	position in the template where the tag is used and no value will be
	returned. If display is false, no output will be printed but the resulting
	string will be given back to the caller.
	*/
	global $g_comvatar;

	$g_comvatar->get_string_type($name, $user_name, $user_email, $user_url);
	$avatar_url = $g_comvatar->get_smart_avatar_url($user_name, $user_email,
		$user_url, $icon_size, $default_avatar_url);
	if (!display)
		return $avatar_url;
	echo $avatar_url;
}

function comvatar_image($name='', $default_avatar_url='', $icon_size='',
	$class_name=COMVATAR_CSS_CLASS, $display=true)
{
	/* Returns an HTML <img> tag of an avatar image in all cases, either
	of a default one, favatar or gravatar. XHTML compliant.

	The function parameter for the comvatar_image template tag are the
	same as the parameter for comvatar_url template tag. See the above
	function comvatar_url for an explanation of the parameter. Additionally
	the following parameter are defined:

	class_name: (string) If set the default CSS class for th <img /> tag will
	overridden to the one defined in this parameter.
	*/
	global $g_comvatar;

	$g_comvatar->get_string_type($name, $user_name, $user_email, $user_url);
	$html = $g_comvatar->get_smart_avatar_image($user_name, $user_email,
		$user_url, $icon_size, $class_name, $default_avatar_url);
	if (!display)
		return $html;
	echo $html;
}

function comvatar_image_link($name='', $default_avatar_url='', $icon_size='',
	$class_name=COMVATAR_CSS_CLASS, $display=true)
{
	/* Returns an HTML <a> tag with a link to the users homepage and an
	embedded avatar image if a user url could be found in all cases, either
	of a default one, favatar or gravatar. XHTML compliant.

	The function parameter for the comvatar_image_link template tag are the
	same as the parameter for comvatar_image template tag. See the above
	function comvatar_url for an explanation of the parameter.
	*/
	global $g_comvatar;

	$g_comvatar->get_string_type($name, $user_name, $user_email, $user_url);
	$html = $g_comvatar->get_smart_avatar_image_link($user_name, $user_email,
		$user_url, $icon_size, $class_name, $default_avatar_url);
	if (!display)
		return $html;
	echo $html;
}

function show_registration_manager_link()
{
	/* Shows a link to registration manager for the current user */
	global $g_comvatar;
	$g_comvatar->show_registration_manager_link();
}

function comvatar_registration_manager()
{
	/* Displays the Registration Manager. Included inside a page, you are
	able to use your own style instead of the admin interface style */
	global $g_comvatar_manager;
	if (!isset($g_comvatar_manager))
		$g_comvatar_manager = new ComvatarManager();
	$g_comvatar_manager->show_page();
}

// --------------------------------------------------------------------------
// plugin initialization
// --------------------------------------------------------------------------

function comvatar_init()
{
	// make the Comvatar instance global
	global $g_comvatar;
	if (!isset($g_comvatar))
		$g_comvatar = new Comvatar();
}

if (function_exists('get_currentuserinfo'))
	// WP 1.5, get_currentuserinfo already loaded
	comvatar_init();
else if (function_exists('add_action'))
	// Need to wait until pluggable functions have been loaded,
	// e.g. the Gravatars plugin
	add_action('init', 'comvatar_init');
?>